#include <xmms/xmmsctrl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <stdio.h>
#include <errno.h>
#include <time.h>
#include "remote_cmd.h"
  
#define BAUDRATE B38400
#define PALMDEVICE "/dev/ttyUSB0"
#define _POSIX_SOURCE 1 /* POSIX compliant source */
  
volatile int STOP=FALSE;
struct termios oldtio;
int fd;

int openport();
void WriteNumber(int);

void nap(int s_time)
{
  struct timespec delay;
  
  delay.tv_sec = 0;
  delay.tv_nsec = s_time * 1000000;

  nanosleep(&delay,NULL);
}

int openport()
{
  int disconnected=0;
  struct termios newtio;
  
  do
  {
    fd = open(PALMDEVICE, O_RDWR | O_NOCTTY);
    if (fd < 0)
    {
      if (!disconnected)
      {
        disconnected = 1;
        printf("Connection Lost\n");
      }
      nap(200);
    }
  } while (fd < 0 && errno == ENODEV);
  
  if (fd <0)
  {
    perror(PALMDEVICE);
    exit(-1);
  }
  
  if (disconnected)
  {
    disconnected = 0;
    printf("Connected\n");
  }
  
  tcgetattr(fd,&oldtio); /* save current port settings */
  
  bzero(&newtio, sizeof(newtio));
  newtio.c_cflag = BAUDRATE | CRTSCTS | CS8 | CLOCAL | CREAD;
  newtio.c_iflag = IGNPAR;
  newtio.c_oflag = 0;
  
  /* set input mode (non-canonical, no echo,...) */
  newtio.c_lflag = 0;
   
  newtio.c_cc[VTIME]    = 6; /* inter-character timer count in 0.1s */
  newtio.c_cc[VMIN]     = 0; /* If !0, timer would wait forever for 1st char */
  
  tcflush(fd, TCIFLUSH);
  tcsetattr(fd,TCSANOW,&newtio);

  return 0;
}

void WriteNumber(int number)
{
  char numstr[NumSize+1];
  
  snprintf(numstr,NumSize+1,"%d",number);
  write(fd,numstr,NumSize+1);
}

int ReadNumber() {
  char NumStr[NumSize + 1]; /* 4 Digit + Null */
  int RCount;

  RCount = read(fd,&NumStr,NumSize + 1);
  if (RCount != NumSize + 1)
    return -1;
  
  NumStr[NumSize] = (char) NULL; /* Just in case string isn't terminated */
  
  return strtol(NumStr, NULL, 10);
}

int main()
{
  char buf, **title = NULL, *title2 = NULL;
  int count, size;
  int ctime,ttime;
  int position;
  int a;
    
  openport();
  
  while (STOP==FALSE)
  {
    if (read(fd,&buf,1) == 0)
    {
      close(fd);
      openport();
      continue;
    }
    switch (buf)
    {
      case CmdPrev:
        printf("Prev\n");
        if (xmms_remote_is_playing(0))
          xmms_remote_playlist_prev(0);
        nap(50);
        break;
      case CmdBack:
        printf("Back\n");
        if (xmms_remote_is_playing(0))
          xmms_remote_jump_to_time(0,xmms_remote_get_output_time(0)-5000);
        break;
      case CmdBack60:
        printf("Back\n");
        if (xmms_remote_is_playing(0))
          xmms_remote_jump_to_time(0,xmms_remote_get_output_time(0)-60000);
        break;
      case CmdPlay:
        printf("Play\n");
        if (xmms_remote_is_playing(0))
          xmms_remote_pause(0);
        else
          xmms_remote_play(0);
        break;
      case CmdForward:
        printf("Forward\n");
        if (xmms_remote_is_playing(0))
          xmms_remote_jump_to_time(0,xmms_remote_get_output_time(0)+5000);
        break;
      case CmdForward60:
        printf("Forward\n");
        if (xmms_remote_is_playing(0))
          xmms_remote_jump_to_time(0,xmms_remote_get_output_time(0)+60000);
        break;
      case CmdNext:
        printf("Next\n");
        if (xmms_remote_is_playing(0))
          xmms_remote_playlist_next(0);
        nap(50);
        break;
      case CmdCheckPlayList:
        if (xmms_remote_is_running(0) == FALSE)
        {
          WriteNumber(-2);
          break;
        }
        
        if (count != xmms_remote_get_playlist_length(0))
        {
          WriteNumber(1);
          break;
        }
        
        for (a=0;a<count;a++)
          if (strcmp(title[a], xmms_remote_get_playlist_title(0,a)) != 0)
          {
            WriteNumber(1);
            break;
          }
        
        /* If we did not break out of the for loop */
        if (a == count)
          WriteNumber(0);
        
        break;
      case CmdGetPlayList:
        printf("Get Play List\n");
        if (xmms_remote_is_running(0) == FALSE)
        {
          WriteNumber(-2);
          break;
        }
        
        if (title != NULL)
        {
          for (a=0;a<count;a++)
            free(title[a]);
          free(title);
        }
        
        count = xmms_remote_get_playlist_length(0);
        WriteNumber(count);
        
        title = malloc(count*sizeof(*title));

        size = 0;
        for (a=0;a<count;a++)
        {
          title[a] = xmms_remote_get_playlist_title(0,a);
          size += strlen(title[a]) + 1;
        }
        WriteNumber(size);

        for (a=0;a<count;a++)
        {
          size = strlen(title[a]);
          write(fd,title[a],size+1);
        }
        break;
      case CmdGetPos:
        if (xmms_remote_is_running(0) == FALSE)
        {
          WriteNumber(-2);
          break;
        }
        WriteNumber(xmms_remote_get_playlist_pos(0));
        break;
      case CmdSetPos:
/*        read(fd,&buf,1);
        printf("Set Position: %d\n", buf);
*/
        position = ReadNumber();
        xmms_remote_set_playlist_pos(0,position);
        nap(50);
        break;
      case CmdGetTime:
        if (xmms_remote_is_running(0) == FALSE)
        {
          WriteNumber(-2);
          break;
        }
        ctime = xmms_remote_get_output_time(0) / 1000;
        a = xmms_remote_get_playlist_pos(0);
        ttime = xmms_remote_get_playlist_time(0,a) / 1000;

        WriteNumber(ctime);
        WriteNumber(ttime);

        break;
      case CmdSetTime:
        ctime = ReadNumber() * 1000;

        a = xmms_remote_get_playlist_pos(0);
        ttime = xmms_remote_get_playlist_time(0,a);

        if (ctime >= 0 && ctime <= ttime)
          xmms_remote_jump_to_time(0,ctime);
        nap(50);
        break;
      case CmdGetTitle:
        printf("Get Play Title\n");
        if (xmms_remote_is_running(0) == FALSE)
        {
          WriteNumber(-2);
          break;
        }
        
        if (title2 != NULL)
          free(title2);
        
        title2 = xmms_remote_get_playlist_title(0,xmms_remote_get_playlist_pos(0));
        
        size = strlen(title2) + 1;
        WriteNumber(size);
        write(fd,title2,size);

        break;
      case CmdGetVol:
        if (xmms_remote_is_running(0) == FALSE)
        {
          WriteNumber(-2);
          break;
        }
        WriteNumber(xmms_remote_get_main_volume(0));
        break;
      case CmdSetVol:
        xmms_remote_set_main_volume(0, ReadNumber());
        nap(50);
        break;
    }
  }
  tcsetattr(fd,TCSANOW,&oldtio);

  return 0;
}
